# Copyright(c) 2018, Intel Corporation
#
# Redistribution  and  use  in source  and  binary  forms,  with  or  without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of  source code  must retain the  above copyright notice,
#   this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name  of Intel Corporation  nor the names of its contributors
#   may be used to  endorse or promote  products derived  from this  software
#   without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,  BUT NOT LIMITED TO,  THE
# IMPLIED WARRANTIES OF  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT  SHALL THE COPYRIGHT OWNER  OR CONTRIBUTORS BE
# LIABLE  FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR
# CONSEQUENTIAL  DAMAGES  (INCLUDING,  BUT  NOT LIMITED  TO,  PROCUREMENT  OF
# SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE,  DATA, OR PROFITS;  OR BUSINESS
# INTERRUPTION)  HOWEVER CAUSED  AND ON ANY THEORY  OF LIABILITY,  WHETHER IN
# CONTRACT,  STRICT LIABILITY,  OR TORT  (INCLUDING NEGLIGENCE  OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
import sys
import uuid

# pylint: disable=E0602, E0603

class TestSysObject(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.system = mockopae.test_system()
        cls.platform = mockopae.test_platform.get("skx-p")
        cls.system.initialize()
        cls.system.prepare_sysfs(cls.platform)
        opae.fpga.initialize(None)
        props = opae.fpga.properties(type=opae.fpga.ACCELERATOR)
        cls.afu_toks = opae.fpga.enumerate([props])
        assert cls.afu_toks
        cls.afu_handle = opae.fpga.open(cls.afu_toks[0])
        assert cls.afu_handle
        props = opae.fpga.properties(type=opae.fpga.DEVICE)
        cls.fme_toks = opae.fpga.enumerate([props])
        assert cls.fme_toks
        cls.fme_handle = opae.fpga.open(cls.fme_toks[0], opae.fpga.OPEN_SHARED)
        assert cls.fme_handle

    @classmethod
    def tearDownClass(cls):
        cls.system.finalize()

    def test_token_object(self):
        afuid = self.afu_toks[0].afu_id.bytes()[:-1]
        afu_guid = self.platform.devices[0].afu_guid
        u1 = uuid.UUID(afuid)
        u2 = uuid.UUID(afu_guid)
        assert u1 == u2
        afuid2 = self.afu_toks[0]["afu_id"].bytes()[:-1]
        assert afuid == afuid2
        self.assertEquals(self.afu_toks[0].wrong, None)
        with self.assertRaises(RuntimeError):
            print(self.afu_toks[0]['../fpga'].read64())

    def test_handle_object(self):
        afuid = self.afu_handle.afu_id.bytes()[:-1]
        afu_guid = self.platform.devices[0].afu_guid
        u1 = uuid.UUID(afuid)
        u2 = uuid.UUID(afu_guid)
        assert u1 == u2

    def test_object_object(self):
        pr = self.fme_handle.pr
        interface_id = pr.interface_id
        u1 = uuid.UUID(interface_id.bytes()[:-1])
        u2 = uuid.UUID(self.platform.devices[0].fme_guid)
        assert u1 == u2

    def test_object_read64(self):
        errors = self.afu_handle.errors
        rev = errors.revision
        n1 = rev.read64()
        self.assertEquals(n1, 1)
        with self.assertRaises(RuntimeError):
            print(errors.read64())





#!/usr/bin/env python
# Copyright(c) 2017, Intel Corporation
#
# Redistribution  and  use  in source  and  binary  forms,  with  or  without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of  source code  must retain the  above copyright notice,
#  this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#  this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name  of Intel Corporation  nor the names of its contributors
#   may be used to  endorse or promote  products derived  from this  software
#   without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,  BUT NOT LIMITED TO,  THE
# IMPLIED WARRANTIES OF  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT  SHALL THE COPYRIGHT OWNER  OR CONTRIBUTORS BE
# LIABLE  FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR
# CONSEQUENTIAL  DAMAGES  (INCLUDING,  BUT  NOT LIMITED  TO,  PROCUREMENT  OF
# SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE,  DATA, OR PROFITS;  OR BUSINESS
# INTERRUPTION)  HOWEVER CAUSED  AND ON ANY THEORY  OF LIABILITY,  WHETHER IN
# CONTRACT,  STRICT LIABILITY,  OR TORT  (INCLUDING NEGLIGENCE  OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import os
import sys
import argparse
import re
import json
import zipfile
import uuid
from afu import AFU

AFU_JSON_MGR_EXEC = "afu_json_mgr"
DESCRIPTION = 'Intel FPGA AFU JSON Manager'

USAGE = """
{0}

{1} <cmd> [options]

The following values for <cmd> are currently supported:
\t help - displays this message
\t create-json - creates a minimal JSON file describing an AFU
\t json-info - extract information from JSON file for use in C or Verilog

{1} <cmd> --h will give command specific help
""".format(DESCRIPTION, AFU_JSON_MGR_EXEC)


# Create an AFU JSON file, filling in a few key fields.
def create_json(subargs):
    # Read the template JSON file
    filepath = os.path.dirname(os.path.realpath(__file__))
    template_path = "schema/afu_template.json"
    afu = AFU()
    if (zipfile.is_zipfile(filepath)):
        archive = zipfile.ZipFile(filepath, 'r')
        afu.load_afu_desc_file_hdl(archive.open(template_path, "r"))
    else:
        afu.load_afu_desc_file_hdl(open(os.path.join(filepath,
                                                     template_path), "r"))

    accel = afu.afu_json['afu-image']['accelerator-clusters'][0]
    accel['name'] = subargs.name

    # Top-level interface specified?
    if (subargs.top_ifc):
        afu.update_afu_json(['afu-image/afu-top-interface/class:' +
                             subargs.top_ifc])

    # Either set the specified UUID or pick one
    if (subargs.uuid):
        accel['accelerator-type-uuid'] = subargs.uuid
    else:
        accel['accelerator-type-uuid'] = str(uuid.uuid1())

    # The output file name is either the AFU name or a specified file path
    json_path = subargs.name + '.json'
    if (subargs.afu_json):
        json_path = subargs.afu_json
    print("Writing {0}".format(json_path))
    with open(json_path, 'w') as a:
        a.write(afu.dumps() + '\n')


def emit_header_comment(f, src):
    f.write('''//
// Generated by afu_json_mgr from {0}
//

'''.format(src))


# Flatten JSON into a simple, single level dictionary to emit as header files
def flatten_json(subargs):
    afu = AFU(subargs.afu_json)

    entries = dict()

    emit_types = (int, bool, str, str, float)
    # Don't emit some keys.  For example, user clock frequency may not be
    # known at compile time, so avoid emitting potentially false information.
    skip_keys = ['clock-frequency-low', 'clock-frequency-high']

    # Flattan all entries in afu-image that are of type emit_types
    image = afu.afu_json['afu-image']
    for k in sorted(image.keys()):
        if (isinstance(image[k], emit_types) and k not in skip_keys):
            tag = str(k).replace('-', '_')
            v = image[k]
            # Does it look like a number?
            if (not re.match('^[0-9.]+$', str(v))):
                v = '"' + str(v) + '"'
            entries['afu_image_' + tag] = v

    # Some special names, taken from other levels
    accel = image['accelerator-clusters'][0]
    entries['afu_accel_name'] = '"' + accel['name'] + '"'
    entries['afu_accel_uuid'] = accel['accelerator-type-uuid']
    try:
        # May not be present.  (Will become required eventually.)
        entries['afu_top_ifc'] = '"' + \
            afu.afu_json['afu-image']['afu-top-interface']['class'] + '"'
    except Exception:
        None

    return entries


# Emit C or Verilog header files based on an AFU JSON file
def json_info(entries, subargs):
    afu = AFU(subargs.afu_json)

    # C header
    if (subargs.c_hdr):
        print("Writing {0}".format(subargs.c_hdr))
        with open(subargs.c_hdr, 'w') as p:
            emit_header_comment(p, subargs.afu_json)
            p.write('#ifndef __AFU_JSON_INFO__\n')
            p.write('#define __AFU_JSON_INFO__\n\n')
            for k in sorted(entries.keys()):
                v = entries[k]
                if (k == 'afu_accel_uuid'):
                    v = '"' + v.upper() + '"'
                p.write('#define {0} {1}\n'.format(k.upper(), v))
            p.write('\n#endif // __AFU_JSON_INFO__\n')

    # Verilog header
    if (subargs.verilog_hdr):
        print("Writing {0}".format(subargs.verilog_hdr))
        with open(subargs.verilog_hdr, 'w') as p:
            emit_header_comment(p, subargs.afu_json)
            p.write('`ifndef __AFU_JSON_INFO__\n')
            p.write('`define __AFU_JSON_INFO__\n\n')
            for k in sorted(entries.keys()):
                v = entries[k]
                if (k == 'afu_accel_uuid'):
                    v = "128'h" + entries[k].replace('-', '_')
                p.write('`define {0} {1}\n'.format(k.upper(), v))
            p.write('\n`endif // __AFU_JSON_INFO__\n')


def run_afu_json_mgr():
    parser = argparse.ArgumentParser(usage=USAGE, add_help=False)
    parser.add_argument("cmd", nargs="?")
    parser.add_argument("remain_args", nargs=argparse.REMAINDER)
    args = parser.parse_args(sys.argv[1:])
    cmd_description = "{0} {1}".format(AFU_JSON_MGR_EXEC, args.cmd)
    subparser = argparse.ArgumentParser(description=cmd_description)
    subparser._optionals.title = 'Options'

    if args.cmd == "help" or not args.cmd:
        print(USAGE)

    elif args.cmd == "create-json":
        subparser.usage = "\n" + cmd_description + \
            " --name=<AFU_NAME> --top-ifc=<TOP_INTERFACE_CLASS>"\
            " --uuid=<AFU_UUID> --afu-json=<OUTPUT_JSON>\n"
        subparser.add_argument('--name', required=True,
                               help='AFU name (REQUIRED)')
        subparser.add_argument('--top-ifc', required=False,
                               default='ccip_std_afu',
                               help='Top-level interface class name. '
                               'Default is ccip_std_afu.  See the output of '
                               '"afu_platform_config --help" for a list of '
                               'top-level interface classes.')
        subparser.add_argument('--uuid', required=False,
                               help='Accelerator UUID (default: chosen at '
                               'random)')
        subparser.add_argument('--afu-json', required=False,
                               help='Output path for JSON file '
                               '(default <afu_name>.json)')
        subargs = subparser.parse_args(args.remain_args)
        create_json(subargs)

    elif args.cmd == "json-info":
        subparser.usage = "\n" + cmd_description + \
            " --afu-json=<INPUT_JSON>"\
            " --c-hdr=<OUTPUT_C_HDR_PATH>"\
            " --verilog-hdr=<OUTPUT_VERILOG_HDR_PATH>\n"
        subparser.add_argument('--afu-json', required=True,
                               help='Input path of JSON file. ')
        subparser.add_argument('--c-hdr', required=False,
                               help='Path of generated C header file. ')
        subparser.add_argument('--verilog-hdr', required=False,
                               help='Path of generated Verilog header file. ')
        subargs = subparser.parse_args(args.remain_args)
        entries = flatten_json(subargs)
        json_info(entries, subargs)

    else:
        raise Exception("{0} is not a command for {1}!".format(
            args.cmd, DESCRIPTION))


def main():
    try:
        sys.exit(run_afu_json_mgr())
    except Exception as e:
        print("ERROR: {0}".format(e.__str__()))
        sys.exit(1)


if __name__ == '__main__':
    main()



# Copyright(c) 2018, Intel Corporation
#
# Redistribution  and  use  in source  and  binary  forms,  with  or  without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of  source code  must retain the  above copyright notice,
#   this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name  of Intel Corporation  nor the names of its contributors
#   may be used to  endorse or promote  products derived  from this  software
#   without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,  BUT NOT LIMITED TO,  THE
# IMPLIED WARRANTIES OF  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT  SHALL THE COPYRIGHT OWNER  OR CONTRIBUTORS BE
# LIABLE  FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR
# CONSEQUENTIAL  DAMAGES  (INCLUDING,  BUT  NOT LIMITED  TO,  PROCUREMENT  OF
# SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE,  DATA, OR PROFITS;  OR BUSINESS
# INTERRUPTION)  HOWEVER CAUSED  AND ON ANY THEORY  OF LIABILITY,  WHETHER IN
# CONTRACT,  STRICT LIABILITY,  OR TORT  (INCLUDING NEGLIGENCE  OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
import os
from setuptools import setup, find_packages
from setuptools.command.build_ext import build_ext
from distutils.extension import Extension


# get the original build_extensions method
original_build_extensions = build_ext.build_extensions


def override_build_extensions(self):
    if '-Wstrict-prototypes' in self.compiler.compiler_so:
        self.compiler.compiler_so.remove('-Wstrict-prototypes')
    self.compiler.compiler_so.append('-fvisibility=hidden')
    # call the original build_extensions
    original_build_extensions(self)


# replace build_extensions with our custom version
build_ext.build_extensions = override_build_extensions


class pybind_include_dirs(object):
    def __init__(self, user=False):
        self.user = user

    def __str__(self):
        import pybind11
        return pybind11.get_include(self.user)


extensions = [
    Extension("opae.fpga._opae",
              sources=["pyproperties.cpp",
                       "pycontext.cpp",
                       "pyhandle.cpp",
                       "pytoken.cpp",
                       "pyshared_buffer.cpp",
                       "pyevents.cpp",
                       "pyerrors.cpp",
                       "pysysobject.cpp",
                       "opae.cpp"],
              language="c++",
              extra_compile_args=["-std=c++11"],
              extra_link_args=["-std=c++11"],
              include_dirs=[
                  "@CMAKE_INSTALL_PREFIX@/include",
                  os.environ.get("OPAE_INCLUDE_DIR", ""),
                  pybind_include_dirs(),
                  pybind_include_dirs(True)
              ],
              libraries=["opae-c", "opae-cxx-core", "uuid"],
              library_dirs=[os.environ.get("OPAE_LIBRARY_DIR", ""),
                            "@CMAKE_INSTALL_PREFIX@/lib",
                            "@CMAKE_INSTALL_PREFIX@/lib64"])
]

setup(
    name="opae.fpga",
    version="@OPAE_VERSION@",
    packages=find_packages(),
    entry_points={
        'console_scripts': [
        ]
    },
    ext_modules=extensions,
    install_requires=['pybind11>=@PYOPAE_PYBIND11_VERSION@'],
    description="pyopae provides Python bindings around the "
                "OPAE C API",
    license="BSD3",
    keywords="OPAE accelerator fpga bindings",
    url="https://01.org/OPAE",
)



# Copyright(c) 2017, Intel Corporation
#
# Redistribution  and  use  in source  and  binary  forms,  with  or  without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of  source code  must retain the  above copyright notice,
#  this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#  this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name  of Intel Corporation  nor the names of its contributors
#   may be used to  endorse or promote  products derived  from this  software
#   without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,  BUT NOT LIMITED TO,  THE
# IMPLIED WARRANTIES OF  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT  SHALL THE COPYRIGHT OWNER  OR CONTRIBUTORS BE
# LIABLE  FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR
# CONSEQUENTIAL  DAMAGES  (INCLUDING,  BUT  NOT LIMITED  TO,  PROCUREMENT  OF
# SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE,  DATA, OR PROFITS;  OR BUSINESS
# INTERRUPTION)  HOWEVER CAUSED  AND ON ANY THEORY  OF LIABILITY,  WHETHER IN
# CONTRACT,  STRICT LIABILITY,  OR TORT  (INCLUDING NEGLIGENCE  OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import json
import os
import shutil
import sys
import utils
import zipfile
from metadata import metadata
from gbs import GBS, GBS_EXT

# Update sys.path to include jsonschema folder from different locations
try:
    # pkgPATH1 : jsonschema search path for opae-sdk/tools/extra/packager
    pkgPath1 = os.path.join(sys.path[0], 'jsonschema-2.3.0')

    # pkgPath2 : current packager script location
    pkgPath2 = os.path.abspath(os.path.dirname(sys.argv[0]))
    dirList = pkgPath2.split("/")
    dirList = dirList[:-1]
    pkgPath2 = "/".join(dirList)

    # pkgPath3 : jsonschema search path for current packager location
    pkgPath3 = pkgPath2 + "/share/opae/python/jsonschema-2.3.0"

    sys.path.append(pkgPath1)
    sys.path.append(pkgPath3)
    from jsonschema import validators
    from jsonschema import exceptions
except ImportError:
    print("jsonschema module has no validatiors() or exceptions()")
    raise

filepath = os.path.dirname(os.path.realpath(__file__))
schema_path = "schema/afu_schema_v01.json"
if(zipfile.is_zipfile(filepath)):
    archive = zipfile.ZipFile(filepath, 'r')
    afu_schema = json.load(archive.open(schema_path, "r"))
else:
    afu_schema = json.load(open(filepath + "/" + schema_path, "r"))

ARCHIVE_FORMAT = "zip"
ARCHIVE_EXT = ".zip"


class AFU(object):
    def __init__(self, afu_desc_file=None):
        self.afu_json = {}
        self.metadata_len = 0
        self.afu_desc_file = afu_desc_file
        if afu_desc_file:
            self.load_afu_desc_file(afu_desc_file)

    @classmethod
    def create_afu_from_gbs(cls, gbs):
        afu = cls()

        afu.afu_json = gbs.gbs_info
        afu.metadata_len = gbs.metadata_len

        return afu

    def load_afu_desc_file(self, afu_desc_file):
        if os.path.exists(afu_desc_file):
            self.afu_desc_file = os.path.abspath(afu_desc_file)
            self.afu_dir = os.path.dirname(afu_desc_file)
        else:
            raise Exception("Cannot find {0}".format(afu_desc_file))

        self.afu_json = json.load(open(self.afu_desc_file, "r"))
        self.compat_update()

        if not self.validate():
            raise Exception("Accelerator description file failed validation!")

    # Load AFU JSON file given an open file handle
    def load_afu_desc_file_hdl(self, afu_desc_file_hdl):
        self.afu_json = json.load(afu_desc_file_hdl)
        self.compat_update()

        if not self.validate():
            raise Exception("Accelerator description file failed validation!")

    # Update/rename fields as needed to maintain backward compatibility
    def compat_update(self):
        try:
            afu_ifc = self.afu_json['afu-image']['afu-top-interface']
            # The interface 'class' used to be called 'name'.
            # Maintain compatibility with older AFUs.
            if ('name' in afu_ifc):
                afu_ifc['class'] = afu_ifc.pop('name')
        except KeyError as e:
            None

    def validate(self, packaging=False):
        if self.afu_json == {}:
            return False
        try:
            validators.validate(self.afu_json, afu_schema)
        except exceptions.ValidationError as ve:
            print("JSON schema error at {0}: {1}".format(
                str(list(ve.path)), str(ve.message)))
            return False

        # If emitting a GBS file do some extra validation beyond the schema.
        if packaging:
            # User clocks can be "auto" in the source JSON in order to
            # set the frequency to the actual achieved speed.  When
            # creating the GBS, the frequencies must be numbers.
            for clock in ['clock-frequency-high', 'clock-frequency-low']:
                if clock in self.afu_json['afu-image']:
                    f = self.afu_json['afu-image'][clock]
                    if not isinstance(f, (int, float)):
                        print("JSON schema error at {0}: {1}").format(
                            "afu-image/" + clock, "expected number")
                        raise Exception("Accelerator description file " +
                                        "failed validation!")

        return True

    def update_afu_json(self, key_values):
        try:
            for value in key_values:
                # Colon separates key and value
                curr_val = value.split(':')
                curr_val[1] = utils.convert_to_native_type(curr_val[1])
                if self.afu_json:
                    # Compatibility support for old scripts that set
                    # interface-uuid, assuming it would be found in
                    # afu-image.  After all scripts are updated, this
                    # check can be removed.
                    if curr_val[0] == 'interface-uuid':
                        curr_val[0] = 'afu-image/interface-uuid'

                    # Slash separates key hierarchy
                    key = curr_val[0].split('/')
                    if len(key) > 1:
                        # Walk key hierarchy
                        afu = self.afu_json
                        for k in key[:-1]:
                            if k not in afu:
                                # Intermediate key not present, add it.
                                afu[k] = dict()
                            afu = afu[k]

                        # Add the new value
                        afu[key[-1]] = curr_val[1]
                    else:
                        # Old method didn't support key hierarchy.  Search
                        # for the key either in afu-image or at top-level.
                        # If not found, assume top-level.
                        if key[0] in self.afu_json:
                            self.afu_json[key[0]] = curr_val[1]
                        elif key[0] in self.afu_json['afu-image']:
                            self.afu_json['afu-image'][key[0]] = curr_val[1]
                        else:
                            self.afu_json[key[0]] = curr_val[1]
        except IndexError as e:
            print(e)
            raise Exception(
                "Invalid <key>:<value> pair passed using --set-value")

        if not self.validate():
            raise Exception(
                'AFU metadata validation failed after updating metadata '
                ' with values provided with --set_value')

    def create_gbs(self, rbf_file, gbs_file, key_values=None):
        if key_values:
            self.update_afu_json(key_values)

        # Set the expected magic number if it hasn't already been set
        if 'magic-no' not in self.afu_json['afu-image']:
            self.afu_json['afu-image']['magic-no'] = 0x1d1f8680

        self.validate(packaging=True)
        gbs = GBS.create_gbs_from_afu_info(rbf_file, self.afu_json)
        return gbs.write_gbs(gbs_file)

    # Dump AFU JSON to string
    def dumps(self):
        return json.dumps(self.afu_json, indent=3)

    def package(self, rbf_file, sw_dir, doc_dir, package_name):
        image_dir = os.path.join(utils.get_work_dir(), "image_0")
        if not os.path.exists(image_dir):
            os.makedirs(image_dir)

        gbs_name = os.path.splitext(
            os.path.basename(
                self.afu_desc_file))[0] + GBS_EXT
        gbs_path = os.path.join(image_dir, gbs_name)
        self.create_gbs(rbf_file, gbs_path)

        shutil.copyfile(
            self.afu_desc_file, os.path.join(
                image_dir, os.path.basename(
                    self.afu_desc_file)))

        package_dir = os.path.join(utils.get_work_dir(), "package")
        shutil.make_archive(
            os.path.join(
                package_dir,
                "image_0"),
            ARCHIVE_FORMAT,
            image_dir)
        if sw_dir:
            shutil.make_archive(
                os.path.join(
                    package_dir,
                    "sw"),
                ARCHIVE_FORMAT,
                sw_dir)
        if doc_dir:
            shutil.make_archive(
                os.path.join(
                    package_dir,
                    "docs"),
                ARCHIVE_FORMAT,
                doc_dir)

        shutil.make_archive(package_name, ARCHIVE_FORMAT, package_dir)
        shutil.rmtree(utils.get_work_dir())



# Copyright(c) 2018, Intel Corporation
#
# Redistribution  and  use  in source  and  binary  forms,  with  or  without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of  source code  must retain the  above copyright notice,
#   this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name  of Intel Corporation  nor the names of its contributors
#   may be used to  endorse or promote  products derived  from this  software
#   without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,  BUT NOT LIMITED TO,  THE
# IMPLIED WARRANTIES OF  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT  SHALL THE COPYRIGHT OWNER  OR CONTRIBUTORS BE
# LIABLE  FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR
# CONSEQUENTIAL  DAMAGES  (INCLUDING,  BUT  NOT LIMITED  TO,  PROCUREMENT  OF
# SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE,  DATA, OR PROFITS;  OR BUSINESS
# INTERRUPTION)  HOWEVER CAUSED  AND ON ANY THEORY  OF LIABILITY,  WHETHER IN
# CONTRACT,  STRICT LIABILITY,  OR TORT  (INCLUDING NEGLIGENCE  OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
import struct
import sys

# pylint: disable=E0602, E0603

class TestSharedBuffer(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.system = mockopae.test_system()
        cls.platform = mockopae.test_platform.get("skx-p")
        cls.system.initialize()
        cls.system.prepare_sysfs(cls.platform)
        opae.fpga.initialize(None)
        cls.props = opae.fpga.properties(type=opae.fpga.ACCELERATOR)
        cls.toks = opae.fpga.enumerate([cls.props])
        assert cls.toks
        cls.handle = opae.fpga.open(cls.toks[0])
        assert cls.handle

    @classmethod
    def tearDownClass(cls):
        cls.system.finalize()

    def test_allocate(self):
        buff1 = opae.fpga.allocate_shared_buffer(self.handle, 4096)
        buff2 = opae.fpga.allocate_shared_buffer(self.handle, 4096)
        assert buff1
        assert buff2
        assert buff1.size() == 4096
        assert buff1.wsid() != 0
        # TODO: look into wsid in new mock system
        # assert buff1.io_address() != 0
        mv = memoryview(buff1)
        assert mv
        assert not buff1.compare(buff2, 4096)
        buff1.fill(0xAA)
        buff2.fill(0xEE)
        assert buff1.compare(buff2, 4096)
        if sys.version_info[0] == 2:
            assert mv[0] == '\xaa'
            assert mv[-1] == '\xaa'
        else:
            assert mv[0] == 0xaa
            assert mv[-1] == 0xaa
        ba = bytearray(buff1)
        assert ba[0] == 0xaa
        buff1[42] = int(65536)
        assert struct.unpack('<L', (bytearray(buff1[42:46])))[0] == 65536

    def test_conext_release(self):
        assert self.handle
        self.handle.close()
        with opae.fpga.open(self.toks[0]) as h:
            buff = opae.fpga.allocate_shared_buffer(h, 4096)
            assert buff
            assert buff.size() == 4096
            assert buff.wsid() != 0
        assert not h
        assert buff.size() == 0
        assert buff.wsid() == 0




from __future__ import unicode_literals
import sys
import operator

try:
    from collections import MutableMapping, Sequence  # noqa
except ImportError:
    from collections.abc import MutableMapping, Sequence  # noqa

PY3 = sys.version_info[0] >= 3

if PY3:
    zip = zip
    from urllib.parse import (
        unquote, urljoin, urlunsplit, SplitResult, urlsplit as _urlsplit
    )
    from urllib.request import urlopen
    str_types = str,
    int_types = int,
    iteritems = operator.methodcaller("items")
else:
    from itertools import izip as zip  # noqa
    from urlparse import (
        urljoin, urlunsplit, SplitResult, urlsplit as _urlsplit  # noqa
    )
    from urllib import unquote  # noqa
    from urllib2 import urlopen  # noqa
    str_types = basestring
    int_types = int, long
    iteritems = operator.methodcaller("iteritems")


# On python < 3.3 fragments are not handled properly with unknown schemes
def urlsplit(url):
    scheme, netloc, path, query, fragment = _urlsplit(url)
    if "#" in path:
        path, fragment = path.split("#", 1)
    return SplitResult(scheme, netloc, path, query, fragment)


def urldefrag(url):
    if "#" in url:
        s, n, p, q, frag = urlsplit(url)
        defrag = urlunsplit((s, n, p, q, ''))
    else:
        defrag = url
        frag = ''
    return defrag, frag


# flake8: noqa



from _opae import (
    properties,
    token,
    handle,
    shared_buffer,
    event,
    enumerate,
    open,
    allocate_shared_buffer,
    register_event,
    error,
    errors,
    sysobject,
    version,
    build,
    memory_barrier)
from _opae import (DEVICE, ACCELERATOR, OPEN_SHARED, EVENT_ERROR,
                   EVENT_INTERRUPT, EVENT_POWER_THERMAL, ACCELERATOR_ASSIGNED,
                   ACCELERATOR_UNASSIGNED, RECONF_FORCE, SYSOBJECT_GLOB)
__all__ = ['properties',
           'token',
           'handle',
           'shared_buffer',
           'event',
           'enumerate',
           'open',
           'allocate_shared_buffer',
           'register_event',
           'error',
           'errors',
           'sysobject',
           'version',
           'build',
           'memory_barrier',
           'DEVICE',
           'ACCELERATOR',
           'OPEN_SHARED',
           'EVENT_ERROR',
           'EVENT_INTERRUPT',
           'EVENT_POWER_THERMAL',
           'ACCELERATOR_ASSIGNED',
           'ACCELERATOR_UNASSIGNED',
           'RECONF_FORCE',
           'SYSOBJECT_GLOB'
           ]



# Copyright(c) 2017, Intel Corporation
#
# Redistribution  and  use  in source  and  binary  forms,  with  or  without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of  source code  must retain the  above copyright notice,
#  this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#  this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name  of Intel Corporation  nor the names of its contributors
#   may be used to  endorse or promote  products derived  from this  software
#   without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,  BUT NOT LIMITED TO,  THE
# IMPLIED WARRANTIES OF  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT  SHALL THE COPYRIGHT OWNER  OR CONTRIBUTORS BE
# LIABLE  FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR
# CONSEQUENTIAL  DAMAGES  (INCLUDING,  BUT  NOT LIMITED  TO,  PROCUREMENT  OF
# SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE,  DATA, OR PROFITS;  OR BUSINESS
# INTERRUPTION)  HOWEVER CAUSED  AND ON ANY THEORY  OF LIABILITY,  WHETHER IN
# CONTRACT,  STRICT LIABILITY,  OR TORT  (INCLUDING NEGLIGENCE  OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import os
import json
import struct
from metadata import constants
from metadata import metadata

RBF_EXT = ".rbf"
GBS_EXT = ".gbs"

"""
Class GBS for operations related to GBS files
"""


class GBS:
    def __init__(self, gbs_file=None):
        self.guid = ''
        self.metadata_len = 0
        self.gbs_info = ''
        self.rbf = ''
        self.metadata = []

        if gbs_file:
            self.filename = os.path.splitext(os.path.basename(gbs_file))[0]
            self.validate_gbs_file(gbs_file)

    """
    classmethod to create a gbs instance from json and
    rbf file. Used to create a new gbs file

    @return instance of the new GBS object
    """
    @classmethod
    def create_gbs_from_afu_info(cls, rbf_file, afu_json):
        gbs = cls()

        rbf = open(rbf_file, 'rb')
        rbf_content = rbf.read()

        gbs.guid = constants.METADATA_GUID
        gbs.metadata_len = len(afu_json)
        gbs.gbs_info = afu_json
        gbs.rbf = rbf_content
        gbs.metadata = metadata.get_metadata(afu_json)
        gbs.filename = os.path.splitext(os.path.basename(rbf_file))[0]

        return gbs

    """
    Set of get methods to retrieve gbs attributes
    """

    def get_gbs_guid(self):
        return self.guid

    def get_gbs_meta_len(self):
        return self.metadata_len

    def get_gbs_info(self):
        return self.gbs_info

    def get_rbf_val(self):
        return self.rbf

    def get_gbs_metadata(self):
        return self.metadata

    """
    Function to print GBS info to the console
    """

    def print_gbs_info(self):
        if self.gbs_info == '':
            raise Exception("No metadata in GBS file")

        print(json.dumps(self.gbs_info, indent=4))

    """
    Function to write a new rbf file to the filesystem
    """

    def write_rbf(self, rbf_file):
        if not rbf_file:
            rbf_file = self.filename + RBF_EXT

        with open(rbf_file, 'wb') as rbf:
            rbf.write(self.rbf)

        return rbf_file

    """
    Function to write a new gbs file to the filesystem
    """

    def write_gbs(self, gbs_file):
        if not gbs_file:
            gbs_file = self.filename + GBS_EXT

        decoded_bytes = [c if isinstance(c, int) else ord(c)
                         for c in self.get_gbs_metadata()]
        gbs_file_header = bytearray(decoded_bytes)

        with open(gbs_file, 'wb') as gbs:
            gbs.write(gbs_file_header + self.rbf)

        return gbs_file

    """
    Function to update gbs info in an object with input info
    """

    def update_gbs_info(self, gbs_info):
        self.gbs_info = gbs_info
        self.metadata = metadata.get_metadata(self.gbs_info)

    """
    Function to make make sure GBS file conforms to standard
    and polpulate the GBS object with appropriate values
    """

    def validate_gbs_file(self, gbs_file):
        file = open(gbs_file, 'rb')
        gbs = file.read()

        if len(constants.METADATA_GUID) >= len(gbs):
            raise Exception("Invalid GBS file")

        self.guid = gbs[:constants.GUID_LEN]
        if self.guid != constants.METADATA_GUID:
            raise Exception("Unsupported GBS format")

        metadata_index = constants.GUID_LEN + constants.SIZEOF_LEN_FIELD

        metadata_len = struct.unpack(
            "<I", gbs[constants.GUID_LEN:metadata_index])
        self.metadata_len = metadata_len[0]

        if self.metadata_len != 0:
            self.gbs_info = json.loads(
                gbs[metadata_index:(metadata_index + self.metadata_len)])

        rbf_index = metadata_index + metadata_len[0]

        if rbf_index == len(gbs):
            raise Exception("No RBF in GBS file!")

        self.rbf = gbs[rbf_index:]

        self.metadata = metadata.get_metadata(self.gbs_info)



import unittest
import packager
import filecmp
import metadata.constants
import struct
import os
import json
from collections import OrderedDict

filepath = os.path.dirname(os.path.realpath(__file__))

afu_json = (
    '{"version": 1, "afu-image": {"clock-frequency-low": 50, '
    '"slot-type-uuid": "166b785b-0a5d-411a-bc2d-5b3d17c479a8"'
    ', "clock-frequency-high": 200, "accelerator-clusters": '
    '[{"total-contexts": 1, "accelerator-type-uuid": '
    '"c000c966-0d82-4272-9aef-fe5f84570612"'
    ', "name": "nlb_400"}], "power": 10}}')

expected_metadata = ("XeonFPGA" + chr(0xb7) + "GBSv001" +
                     struct.pack('<I', len(afu_json)) +
                     afu_json)

expected_no_json_metadata = ("XeonFPGA" + chr(0xb7) + "GBSv001" +
                             struct.pack('<I', 0))


""" Unit tests for the metadata.py file
"""


class TestMetadata(unittest.TestCase):

    def setUp(self):
        self.afu_json = json.loads(afu_json, object_pairs_hook=OrderedDict)

    # compare generated and expected metadata
    def test_get_metadata(self):
        self.assertTrue(
            list(expected_metadata) == metadata.metadata.get_metadata(
                self.afu_json))

    # test the case where there is no JSON
    def test_no_json(self):
        self.assertTrue(list(expected_no_json_metadata) ==
                        metadata.metadata.get_metadata({}))


if __name__ == '__main__':
    unittest.main()



# Copyright(c) 2017, Intel Corporation
#
# Redistribution  and  use  in source  and  binary  forms,  with  or  without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of  source code  must retain the  above copyright notice,
#  this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#  this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name  of Intel Corporation  nor the names of its contributors
#   may be used to  endorse or promote  products derived  from this  software
#   without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,  BUT NOT LIMITED TO,  THE
# IMPLIED WARRANTIES OF  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT  SHALL THE COPYRIGHT OWNER  OR CONTRIBUTORS BE
# LIABLE  FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR
# CONSEQUENTIAL  DAMAGES  (INCLUDING,  BUT  NOT LIMITED  TO,  PROCUREMENT  OF
# SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE,  DATA, OR PROFITS;  OR BUSINESS
# INTERRUPTION)  HOWEVER CAUSED  AND ON ANY THEORY  OF LIABILITY,  WHETHER IN
# CONTRACT,  STRICT LIABILITY,  OR TORT  (INCLUDING NEGLIGENCE  OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import json
from . import constants
import struct
from collections import OrderedDict

"""  This is the entrypoint for generating the metadata
"""


def get_metadata(afu_json):
    ret_list = []

    ret_list.extend(list(constants.METADATA_GUID))

    if len(afu_json) != 0:
        ret_list.extend(pack_int_to_buf(len(json.dumps(afu_json))))
        ret_list.extend(list(json.dumps(afu_json)))

    else:
        ret_list.extend(pack_int_to_buf(0))

    return ret_list


def pack_int_to_buf(val):
    pack_format = "<I"
    buf = struct.pack(pack_format, val)
    return list(buf)



from distutils.core import setup

from jsonschema import __version__


with open("README.rst") as readme:
    long_description = readme.read()


classifiers = [
    "Development Status :: 5 - Production/Stable",
    "Intended Audience :: Developers",
    "License :: OSI Approved :: MIT License",
    "Operating System :: OS Independent",
    "Programming Language :: Python",
    "Programming Language :: Python :: 2",
    "Programming Language :: Python :: 2.6",
    "Programming Language :: Python :: 2.7",
    "Programming Language :: Python :: 3",
    "Programming Language :: Python :: 3.1",
    "Programming Language :: Python :: 3.2",
    "Programming Language :: Python :: 3.3",
    "Programming Language :: Python :: Implementation :: CPython",
    "Programming Language :: Python :: Implementation :: PyPy",
]


setup(
    name="jsonschema",
    version=__version__,
    packages=["jsonschema", "jsonschema.tests"],
    package_data={'jsonschema': ['schemas/*.json']},
    author="Julian Berman",
    author_email="Julian@GrayVines.com",
    classifiers=classifiers,
    description="An implementation of JSON Schema validation for Python",
    license="MIT",
    long_description=long_description,
    url="http://github.com/Julian/jsonschema",
)



#!/usr/bin/env python
# Copyright(c) 2017, Intel Corporation
#
# Redistribution  and  use  in source  and  binary  forms,  with  or  without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of  source code  must retain the  above copyright notice,
#  this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#  this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name  of Intel Corporation  nor the names of its contributors
#   may be used to  endorse or promote  products derived  from this  software
#   without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,  BUT NOT LIMITED TO,  THE
# IMPLIED WARRANTIES OF  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT  SHALL THE COPYRIGHT OWNER  OR CONTRIBUTORS BE
# LIABLE  FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR
# CONSEQUENTIAL  DAMAGES  (INCLUDING,  BUT  NOT LIMITED  TO,  PROCUREMENT  OF
# SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE,  DATA, OR PROFITS;  OR BUSINESS
# INTERRUPTION)  HOWEVER CAUSED  AND ON ANY THEORY  OF LIABILITY,  WHETHER IN
# CONTRACT,  STRICT LIABILITY,  OR TORT  (INCLUDING NEGLIGENCE  OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import os
import sys
import subprocess
import shutil
import argparse
import json
import utils
from afu import AFU
from gbs import GBS
from metadata import metadata


PACKAGER_EXEC = "packager"
DESCRIPTION = 'Intel OPAE FPGA Packager'
VERSION = "@OPAE_VERSION@"

try:
    assert sys.version_info >= (2, 7)
except AssertionError:
    print(DESCRIPTION + " requires Python 2 version 2.7+")
    sys.exit(1)

USAGE = """
{0}

{1} <cmd> [options]

The following values for <cmd> are currently supported:
\t help - displays this message
\t create-gbs - creates GBS file from RBF and Accelerator Description File
\t modify-gbs - modify metadata of existing GBS file using --set-value
\t gbs-info - prints information about GBS file
\t get-rbf - creates RBF file by extracting RBF from GBS file

{1} <cmd> --h will give command specific help
""".format(DESCRIPTION, PACKAGER_EXEC)


def run_packager():
    parser = argparse.ArgumentParser(usage=USAGE, add_help=False)
    parser.add_argument("cmd", nargs="?")
    parser.add_argument("remain_args", nargs=argparse.REMAINDER)
    args = parser.parse_args(sys.argv[1:])
    cmd_description = "{0} {1}".format(PACKAGER_EXEC, args.cmd)
    subparser = argparse.ArgumentParser(description=cmd_description)
    subparser._optionals.title = 'Options'

    if args.cmd == "help" or not args.cmd:
        print(USAGE)

    elif args.cmd == "version":
        if VERSION.startswith("@"):
            try:
                devnull = open(os.devnull, 'w')
                repo = subprocess.check_output('git remote -v',
                                               shell=True,
                                               stderr=devnull)
                version = (subprocess.check_output('git describe --tags',
                                                   shell=True,
                                                   stderr=devnull).split()[0]
                           if "opae-sdk" in repo else "UNKNOWN REPO")
            except subprocess.CalledProcessError:
                version = "UNKNOWN"
        else:
            version = VERSION
        print("{0}: version {1}".format(DESCRIPTION, version))
    elif args.cmd == "create-gbs":
        subparser.usage = "\n" + cmd_description + \
            " --rbf=<RBF_PATH> --afu-json=<AFU_JSON_PATH>"\
            " --gbs=<NAME_FOR_GBS> --set-value <key>:<value>\n"
        subparser.add_argument('--rbf', required=True,
                               help='RBF file (REQUIRED)')
        subparser.add_argument('--afu-json', required=False,
                               help='AFU JSON file that contains metadata')
        subparser.add_argument('--no-metadata', default=False,
                               action='store_true',
                               help='Empty metadata section will be appended')
        subparser.add_argument('--gbs', required=False,
                               help='Output location for gbs file. '
                               'Default is <rbf_name>.gbs')
        subparser.add_argument('--set-value', required=False, nargs='*',
                               help='set values for keys in JSON metadata as '
                               '<key>:<value>. Can be followed by more than '
                               'one <key>:<value> pairs.')
        subargs = subparser.parse_args(args.remain_args)
        afu = AFU(subargs.afu_json)
        gbs_file = afu.create_gbs(subargs.rbf, subargs.gbs, subargs.set_value)
        print("Wrote {0}".format(gbs_file))

    elif args.cmd == "modify-gbs":
        subparser.usage = "\n" + cmd_description + \
            " --input-gbs=<PATH_TO_GBS_TO_BE_MODIFIED>"\
            " --output-gbs=<NAME_FOR_NEW_GBS> --set-value <key>:<value>\n"
        subparser.add_argument('--input-gbs', required=True,
                               help='Path to input gbs file')
        subparser.add_argument('--output-gbs', required=False,
                               help='Path to output gbs file. Will replace '
                               'original file if not provided')
        subparser.add_argument('--set-value', required=True, nargs='*',
                               help='set values for keys in JSON metadata as '
                               '<key>:<value>. Can be followed by more than '
                               'one <key>:<value> pairs.')
        subargs = subparser.parse_args(args.remain_args)
        gbs = GBS(subargs.input_gbs)
        afu = AFU.create_afu_from_gbs(gbs)
        afu.update_afu_json(subargs.set_value)
        afu.validate(packaging=True)
        gbs.update_gbs_info(afu.afu_json)
        gbs_file = gbs.write_gbs(subargs.output_gbs)
        print("Wrote {0}".format(gbs_file))

    elif args.cmd == "package":
        subparser.usage = "\n" + cmd_description + \
            " --afu-json=<AFU_JSON_PATH> --rbf=<RBF_PATH>"\
            " --out=<NAME_OF_PACKAGE>\n"
        subparser.usage += cmd_description + \
            " --afu-json=<AFU_JSON_PATH> --rbf=<RBF_PATH> --sw-dir=<SW_DIR>"\
            " --doc-dir=<DOC_DIR>"
        subparser.add_argument('--afu-json', required=True,
                               help='AFU JSON file that contains metadata '
                               '(REQUIRED)')
        subparser.add_argument('--rbf', required=True,
                               help='RBF file (REQUIRED)')
        subparser.add_argument('--sw-dir', required=False,
                               help='Location of software files to include')
        subparser.add_argument('--doc-dir', required=False,
                               help='Location of documentation files to '
                               'include')
        subparser.add_argument('--out', required=False, default="afu",
                               help='Used to specify name of package')
        subargs = subparser.parse_args(args.remain_args)
        afu = AFU(subargs.afu_json)
        afu.package(subargs.rbf, subargs.sw_dir, subargs.doc_dir, subargs.out)
        print("Wrote {0}.zip".format(subargs.out))

    elif args.cmd == "gbs-info":
        subparser.usage = "\n" + cmd_description + " --gbs=<GBS_PATH>"
        subparser.add_argument('--gbs', required=True,
                               help='Path to GBS file')
        subargs = subparser.parse_args(args.remain_args)
        gbs = GBS(subargs.gbs)
        gbs.print_gbs_info()

    elif args.cmd == "get-rbf":
        subparser.usage = "\n" + cmd_description + \
            "--gbs=<GBS_PATH> --rbf=<NAME_FOR_RBF>"
        subparser.add_argument('--gbs', required=True,
                               help='Path to GBS file from which rbf is to be '
                               'extracted')
        subparser.add_argument('--rbf', required=False,
                               help='Output location for rbf file. Default is '
                               '<gbs_name>.rbf')
        subargs = subparser.parse_args(args.remain_args)
        gbs = GBS(subargs.gbs)
        rbf_file = gbs.write_rbf(subargs.rbf)
        print("Wrote {0}".format(rbf_file))

    else:
        raise Exception("{0} is not a command for {1}!".format(
            args.cmd, DESCRIPTION))


def main():
    try:
        sys.exit(run_packager())
    except Exception as e:
        print("ERROR: {0}".format(e.__str__()))
        sys.exit(1)


if __name__ == '__main__':
    main()



# Copyright(c) 2018, Intel Corporation
#
# Redistribution  and  use  in source  and  binary  forms,  with  or  without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of  source code  must retain the  above copyright notice,
#   this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name  of Intel Corporation  nor the names of its contributors
#   may be used to  endorse or promote  products derived  from this  software
#   without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,  BUT NOT LIMITED TO,  THE
# IMPLIED WARRANTIES OF  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT  SHALL THE COPYRIGHT OWNER  OR CONTRIBUTORS BE
# LIABLE  FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR
# CONSEQUENTIAL  DAMAGES  (INCLUDING,  BUT  NOT LIMITED  TO,  PROCUREMENT  OF
# SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE,  DATA, OR PROFITS;  OR BUSINESS
# INTERRUPTION)  HOWEVER CAUSED  AND ON ANY THEORY  OF LIABILITY,  WHETHER IN
# CONTRACT,  STRICT LIABILITY,  OR TORT  (INCLUDING NEGLIGENCE  OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
import uuid
import unittest
NLB0 = "d8424dc4-a4a3-c413-f89e-433683f9040b"

# pylint: disable=E0602, E0603

class TestProperties(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.system = mockopae.test_system()
        cls.platform = mockopae.test_platform.get("skx-p")
        cls.system.initialize()
        cls.system.prepare_sysfs(cls.platform)
        opae.fpga.initialize(None)

    @classmethod
    def tearDownClass(cls):
        cls.system.finalize()

    def test_set_parent(self):
        props = opae.fpga.properties(type=opae.fpga.DEVICE)
        toks = opae.fpga.enumerate([props])
        assert len(toks) > 0
        props2 = opae.fpga.properties(type=opae.fpga.ACCELERATOR,
                                      parent=toks[0])
        assert props2.parent
        props2 = opae.fpga.properties(type=opae.fpga.ACCELERATOR)
        props2.parent = toks[0]
        assert props2.parent

    def test_guid(self):
        props = opae.fpga.properties(guid=NLB0)
        guid_str = props.guid
        guid = uuid.UUID(guid_str)
        assert str(guid).lower() == NLB0
        props = opae.fpga.properties()
        props.guid = NLB0
        guid_str = props.guid
        guid = uuid.UUID(guid_str)
        assert str(guid).lower() == NLB0

    def test_set_objtype_accelerator(self):
        props = opae.fpga.properties(type=opae.fpga.ACCELERATOR)
        assert props.type == opae.fpga.ACCELERATOR
        props = opae.fpga.properties(type=opae.fpga.DEVICE)
        props.type = opae.fpga.ACCELERATOR
        assert props.type == opae.fpga.ACCELERATOR

    def test_set_objtype_device(self):
        props = opae.fpga.properties(type=opae.fpga.DEVICE)
        assert props.type == opae.fpga.DEVICE
        props = opae.fpga.properties(type=opae.fpga.ACCELERATOR)
        props.type = opae.fpga.DEVICE
        assert props.type == opae.fpga.DEVICE

    def test_set_segment(self):
        props = opae.fpga.properties(segment=0x9090)
        assert props.segment == 0x9090
        props.segment = 0xA1A1
        assert props.segment == 0xA1A1

    def test_set_bus(self):
        props = opae.fpga.properties(bus=0x5e)
        assert props.bus == 0x5e
        props.bus = 0xbe
        assert props.bus == 0xbe

    def test_set_device(self):
        props = opae.fpga.properties(device=0xe)
        assert props.device == 0xe
        props.device = 0xf
        assert props.device == 0xf

    def test_set_function(self):
        props = opae.fpga.properties(function=0x7)
        assert props.function == 0x7
        props.function = 0x6
        assert props.function == 0x6

    def test_set_socket_id(self):
        props = opae.fpga.properties(socket_id=1)
        assert props.socket_id == 1
        props.socket_id = 0
        assert props.socket_id == 0

    def test_set_object_id(self):
        props = opae.fpga.properties(object_id=0xcafe)
        assert props.object_id == 0xcafe
        props.object_id = 0xfade
        assert props.object_id == 0xfade

    def test_set_num_errors(self):
        props = opae.fpga.properties(num_errors=8)
        assert props.num_errors == 8
        props.num_errors = 4
        assert props.num_errors == 4

    def test_set_num_slots(self):
        props = opae.fpga.properties(type=opae.fpga.DEVICE,
                                     num_slots=3)
        assert props.num_slots == 3
        props.num_slots = 2
        assert props.num_slots == 2

    def test_set_bbs_id(self):
        props = opae.fpga.properties(type=opae.fpga.DEVICE,
                                     bbs_id=0xc0c0cafe)
        assert props.bbs_id == 0xc0c0cafe
        props.bbs_id = 0xb0b0fade
        assert props.bbs_id == 0xb0b0fade

    def test_set_bbs_version(self):
        props = opae.fpga.properties(type=opae.fpga.DEVICE,
                                     bbs_version=(0, 1, 2))
        assert props.bbs_version == (0, 1, 2)
        props.bbs_version = (1, 2, 3)
        assert props.bbs_version == (1, 2, 3)

    def test_set_vendor_id(self):
        props = opae.fpga.properties(vendor_id=0xfafa)
        assert props.vendor_id == 0xfafa
        props.vendor_id = 0xdada
        assert props.vendor_id == 0xdada

    def test_set_device_id(self):
        props = opae.fpga.properties(device_id=0xfa)
        assert props.device_id == 0xfa
        props.device_id = 0xda
        assert props.device_id == 0xda

    @unittest.skip("model not implemented yet")
    def test_set_model(self):
        props = opae.fpga.properties(model="intel skxp")
        assert props.model == "intel skxp"
        props.model = "intel skxp 2"
        assert props.model == "intel skxp 2"

    @unittest.skip("local_memory_size not implemented yet")
    def test_set_local_memory_size(self):
        props = opae.fpga.properties(local_memory_size=0xffff)
        assert props.local_memory_size == 0xffff
        props.local_memory_size = 0xaaaa
        assert props.local_memory_size == 0xaaaa

    @unittest.skip("capabilities not implemented yet")
    def test_set_capabilities(self):
        props = opae.fpga.properties(capabilities=0xdeadbeef)
        assert props.capabilities == 0xdeadbeef
        props.capabilities = 0xfeebdaed
        assert props.capabilities == 0xfeebdaed

    def test_set_num_mmio(self):
        props = opae.fpga.properties(type=opae.fpga.ACCELERATOR,
                                     num_mmio=4)
        assert props.num_mmio == 4
        props.num_mmio = 5
        assert props.num_mmio == 5

    def test_set_num_interrupts(self):
        props = opae.fpga.properties(type=opae.fpga.ACCELERATOR,
                                     num_interrupts=9)
        assert props.num_interrupts == 9
        props.num_interrupts = 8
        assert props.num_interrupts == 8

    def test_set_accelerator_state(self):
        props = opae.fpga.properties(
            type=opae.fpga.ACCELERATOR,
            accelerator_state=opae.fpga.ACCELERATOR_ASSIGNED)
        assert props.accelerator_state == opae.fpga.ACCELERATOR_ASSIGNED
        props.accelerator_state = opae.fpga.ACCELERATOR_UNASSIGNED
        assert props.accelerator_state == opae.fpga.ACCELERATOR_UNASSIGNED





import unittest


class HelloWorldTestCase(unittest.TestCase):
    def runTest(self):
        import packager
        self.assertTrue('main' in dir(packager))


if __name__ == '__main__':
    unittest.main()



# test_afu_platform.py
import unittest
import packager
import os
from afu import AFU

filepath = os.path.dirname(os.path.realpath(__file__))


class jsonTest(unittest.TestCase):
    def testGoodAFU(self):
        afu = AFU(filepath + "/test_data/good_afu_test.json")
        self.assertTrue(afu.validate())

    def testEmptyAFU(self):
        try:
            AFU(None)
        except Exception:
            self.fail(
                "AFU() should not throw exception if no AFU JSON is specified")

    def testBadAFU(self):
        self.assertRaises(
            Exception,
            AFU,
            filepath +
            "/test_data/bad_afu_test.json")


if __name__ == '__main__':
    unittest.main()



# Copyright(c) 2018, Intel Corporation
#
# Redistribution  and  use  in source  and  binary  forms,  with  or  without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of  source code  must retain the  above copyright notice,
#   this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name  of Intel Corporation  nor the names of its contributors
#   may be used to  endorse or promote  products derived  from this  software
#   without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,  BUT NOT LIMITED TO,  THE
# IMPLIED WARRANTIES OF  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT  SHALL THE COPYRIGHT OWNER  OR CONTRIBUTORS BE
# LIABLE  FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR
# CONSEQUENTIAL  DAMAGES  (INCLUDING,  BUT  NOT LIMITED  TO,  PROCUREMENT  OF
# SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE,  DATA, OR PROFITS;  OR BUSINESS
# INTERRUPTION)  HOWEVER CAUSED  AND ON ANY THEORY  OF LIABILITY,  WHETHER IN
# CONTRACT,  STRICT LIABILITY,  OR TORT  (INCLUDING NEGLIGENCE  OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
import json
import select
import struct
import subprocess
import threading
import time
import unittest
import uuid
import sys
import opae.fpga

NLB0 = "d8424dc4-a4a3-c413-f89e-433683f9040b"

MOCK_PORT_ERROR = "/tmp/class/fpga/intel-fpga-dev.0/intel-fpga-port.0/errors/errors"

NLB0_MDATA = {"version": 640,
              "afu-image": {"clock-frequency-high": 312,
                            "clock-frequency-low": 156,
                            "power": 50,
                            "interface-uuid": "1a422218-6dba-448e-b302-425cbcde1406",
                            "magic-no": 488605312,
                            "accelerator-clusters": [{"total-contexts": 1,
                                                      "name": "nlb_400",
                                                      "accelerator-type-uuid": "d8424dc4-a4a3-c413-f89e-433683f9040b"}]},
              "platform-name": "MCP"}

class TestProperties(unittest.TestCase):
    def test_set_parent(self):
        props = opae.fpga.properties(type=opae.fpga.DEVICE)
        toks = opae.fpga.enumerate([props])
        props2 = opae.fpga.properties(type=opae.fpga.ACCELERATOR,
                                      parent=toks[0])
        assert props2.parent
        props2 = opae.fpga.properties(type=opae.fpga.ACCELERATOR)
        props2.parent = toks[0]
        assert props2.parent

    def test_guid(self):
        props = opae.fpga.properties(guid=NLB0)
        guid_str = props.guid
        guid = uuid.UUID(guid_str)
        assert str(guid).lower() == NLB0
        props = opae.fpga.properties()
        props.guid = NLB0
        guid_str = props.guid
        guid = uuid.UUID(guid_str)
        assert str(guid).lower() == NLB0

    def test_set_objtype_accelerator(self):
        props = opae.fpga.properties(type=opae.fpga.ACCELERATOR)
        assert props.type == opae.fpga.ACCELERATOR
        props = opae.fpga.properties(type=opae.fpga.DEVICE)
        props.type = opae.fpga.ACCELERATOR
        assert props.type == opae.fpga.ACCELERATOR

    def test_set_objtype_device(self):
        props = opae.fpga.properties(type=opae.fpga.DEVICE)
        assert props.type == opae.fpga.DEVICE
        props = opae.fpga.properties(type=opae.fpga.ACCELERATOR)
        props.type = opae.fpga.DEVICE
        assert props.type == opae.fpga.DEVICE

    def test_set_segment(self):
        props = opae.fpga.properties(segment=0x9090)
        assert props.segment == 0x9090
        props.segment = 0xA1A1
        assert props.segment == 0xA1A1

    def test_set_bus(self):
        props = opae.fpga.properties(bus=0x5e)
        assert props.bus == 0x5e
        props.bus = 0xbe
        assert props.bus == 0xbe

    def test_set_device(self):
        props = opae.fpga.properties(device=0xe)
        assert props.device == 0xe
        props.device = 0xf
        assert props.device == 0xf

    def test_set_function(self):
        props = opae.fpga.properties(function=0x7)
        assert props.function == 0x7
        props.function = 0x6
        assert props.function == 0x6

    def test_set_socket_id(self):
        props = opae.fpga.properties(socket_id=1)
        assert props.socket_id == 1
        props.socket_id = 0
        assert props.socket_id == 0

    def test_set_object_id(self):
        props = opae.fpga.properties(object_id=0xcafe)
        assert props.object_id == 0xcafe
        props.object_id = 0xfade
        assert props.object_id == 0xfade

    def test_set_num_errors(self):
        props = opae.fpga.properties(num_errors=8)
        assert props.num_errors == 8
        props.num_errors = 4
        assert props.num_errors == 4

    def test_set_num_slots(self):
        props = opae.fpga.properties(type=opae.fpga.DEVICE,
                                     num_slots=3)
        assert props.num_slots == 3
        props.num_slots = 2
        assert props.num_slots == 2

    def test_set_bbs_id(self):
        props = opae.fpga.properties(type=opae.fpga.DEVICE,
                                     bbs_id=0xc0c0cafe)
        assert props.bbs_id == 0xc0c0cafe
        props.bbs_id = 0xb0b0fade
        assert props.bbs_id == 0xb0b0fade

    def test_set_bbs_version(self):
        props = opae.fpga.properties(type=opae.fpga.DEVICE,
                                     bbs_version=(0, 1, 2))
        assert props.bbs_version == (0, 1, 2)
        props.bbs_version = (1, 2, 3)
        assert props.bbs_version == (1, 2, 3)

    def test_set_vendor_id(self):
        props = opae.fpga.properties(vendor_id=0xfafa)
        assert props.vendor_id == 0xfafa
        props.vendor_id = 0xdada
        assert props.vendor_id == 0xdada

    def test_set_device_id(self):
        props = opae.fpga.properties(device_id=0xfa)
        assert props.device_id == 0xfa
        props.device_id = 0xda
        assert props.device_id == 0xda

    @unittest.skip("model not implemented yet")
    def test_set_model(self):
        props = opae.fpga.properties(model="intel skxp")
        assert props.model == "intel skxp"
        props.model = "intel skxp 2"
        assert props.model == "intel skxp 2"

    @unittest.skip("local_memory_size not implemented yet")
    def test_set_local_memory_size(self):
        props = opae.fpga.properties(local_memory_size=0xffff)
        assert props.local_memory_size == 0xffff
        props.local_memory_size = 0xaaaa
        assert props.local_memory_size == 0xaaaa

    @unittest.skip("capabilities not implemented yet")
    def test_set_capabilities(self):
        props = opae.fpga.properties(capabilities=0xdeadbeef)
        assert props.capabilities == 0xdeadbeef
        props.capabilities = 0xfeebdaed
        assert props.capabilities == 0xfeebdaed

    def test_set_num_mmio(self):
        props = opae.fpga.properties(type=opae.fpga.ACCELERATOR,
                                     num_mmio=4)
        assert props.num_mmio == 4
        props.num_mmio = 5
        assert props.num_mmio == 5

    def test_set_num_interrupts(self):
        props = opae.fpga.properties(type=opae.fpga.ACCELERATOR,
                                     num_interrupts=9)
        assert props.num_interrupts == 9
        props.num_interrupts = 8
        assert props.num_interrupts == 8

    def test_set_accelerator_state(self):
        props = opae.fpga.properties(
            type=opae.fpga.ACCELERATOR,
            accelerator_state=opae.fpga.ACCELERATOR_ASSIGNED)
        assert props.accelerator_state == opae.fpga.ACCELERATOR_ASSIGNED
        props.accelerator_state = opae.fpga.ACCELERATOR_UNASSIGNED
        assert props.accelerator_state == opae.fpga.ACCELERATOR_UNASSIGNED


class TestToken(unittest.TestCase):
    def test_enumerate(self):
        props = opae.fpga.properties(guid=NLB0)
        toks = opae.fpga.enumerate([props])
        assert toks

    def test_enumerate_kwargs(self):
        toks = opae.fpga.enumerate(guid=NLB0)
        assert toks

    def test_enumerate_empty_kwargs(self):
        toks = opae.fpga.enumerate()
        assert toks

    def test_token_properties(self):
        props = opae.fpga.properties(guid=NLB0)
        toks = opae.fpga.enumerate([props])
        assert toks
        for tok in toks:
            token_props = opae.fpga.properties(tok)
            assert token_props
            assert token_props.guid and token_props.guid != ""


class TestHandle(unittest.TestCase):
    def setUp(self):
        self.props = opae.fpga.properties(type=opae.fpga.ACCELERATOR)
        self.toks = opae.fpga.enumerate([self.props])
        assert self.toks
        self.handle = opae.fpga.open(self.toks[0])
        assert self.handle
        with open('m0.gbs', 'w+b') as fd:
            if sys.version_info[0] == 3:
                fd.write(bytes("XeonFPGA\b7GBSv001\53\02\00\00", 'UTF-8'))
                fd.write(bytes(json.dumps(NLB0_MDATA), 'UTF-8'))
            else:
                fd.write("XeonFPGA\b7GBSv001\53\02\00\00")
                fd.write(json.dumps(NLB0_MDATA))


    def test_open_null_token(self):
        with self.assertRaises(ValueError):
            hndl = opae.fpga.open(None)

    def test_open(self):
        assert self.handle

    def test_reconfigure(self):
        with open('m0.gbs', 'r') as fd:
            self.handle.reconfigure(0, fd)

    def test_close(self):
        self.handle.close()
        assert not self.handle

    def test_context(self):
        self.handle.close()
        assert not self.handle
        with opae.fpga.open(self.toks[0]) as h:
            assert h
        assert not h

    def test_reset(self):
        self.handle.reset()

    def test_close_reset(self):
        self.handle.close()
        assert not self.handle
        with self.assertRaises(RuntimeError):
            self.handle.reset()

    def test_mmio(self):
        offset = 0x100
        write_value = 10
        self.handle.write_csr32(offset, write_value)
        read_value = self.handle.read_csr32(offset)
        assert read_value == write_value
        self.handle.write_csr64(offset, write_value)
        read_value = self.handle.read_csr64(offset)
        assert read_value == write_value
        write_value = 0
        self.handle.write_csr32(offset, write_value)
        read_value = self.handle.read_csr32(offset)
        assert read_value == write_value
        self.handle.write_csr64(offset, write_value)
        read_value = self.handle.read_csr64(offset)
        assert read_value == write_value

    def test_close_mmio(self):
        self.handle.close()
        assert not self.handle
        with self.assertRaises(RuntimeError):
            self.handle.write_csr32(0x100, 0xbeef)

        with self.assertRaises(RuntimeError):
            self.handle.write_csr64(0x100, 0xbeef)

        with self.assertRaises(RuntimeError):
            self.handle.read_csr32(0x100)

        with self.assertRaises(RuntimeError):
            self.handle.read_csr64(0x100)


class TestSharedBuffer(unittest.TestCase):
    def setUp(self):
        self.props = opae.fpga.properties(type=opae.fpga.ACCELERATOR)
        self.toks = opae.fpga.enumerate([self.props])
        assert self.toks
        self.handle = opae.fpga.open(self.toks[0])
        assert self.handle

    def test_allocate(self):
        buff1 = opae.fpga.allocate_shared_buffer(self.handle, 4096)
        buff2 = opae.fpga.allocate_shared_buffer(self.handle, 4096)
        assert buff1
        assert buff2
        assert buff1.size() == 4096
        assert buff1.wsid() != 0
        assert buff1.io_address() != 0
        mv = memoryview(buff1)
        assert mv
        assert not buff1.compare(buff2, 4096)
        buff1.fill(0xAA)
        buff2.fill(0xEE)
        assert buff1.compare(buff2, 4096)
        if sys.version_info[0] == 2:
            assert mv[0] == '\xaa'
            assert mv[-1] == '\xaa'
        else:
            assert mv[0] == 0xaa
            assert mv[-1] == 0xaa
        ba = bytearray(buff1)
        assert ba[0] == 0xaa
        buff1[42] = int(65536)
        assert struct.unpack('<L', (bytearray(buff1[42:46])))[0] == 65536

    def test_conext_release(self):
        assert self.handle
        self.handle.close()
        with opae.fpga.open(self.toks[0]) as h:
            buff = opae.fpga.allocate_shared_buffer(h, 4096)
            assert buff
            assert buff.size() == 4096
            assert buff.wsid() != 0
        assert not h
        assert buff.size() == 0
        assert buff.wsid() == 0

def trigger_port_error(value=1):
    with open(MOCK_PORT_ERROR, 'w') as fd:
        fd.write('0\n')
    if value:
        with open(MOCK_PORT_ERROR, 'w') as fd:
            fd.write('{}\n'.format(value))

class TestEvent(unittest.TestCase):
    def setUp(self):
        trigger_port_error(0)
        self.props = opae.fpga.properties(type=opae.fpga.ACCELERATOR,
                                          socket_id=0)
        self.toks = opae.fpga.enumerate([self.props])
        assert self.toks
        self.handle = opae.fpga.open(self.toks[0])
        assert self.handle


    def tearDown(self):
        trigger_port_error(0)

    def test_events(self):
        err_ev = opae.fpga.register_event(self.handle,
                                          opae.fpga.EVENT_ERROR)
        assert err_ev
        # FIXME: os_object returns long
        # WA is to cast it to int
        os_object = int(err_ev.os_object())
        assert isinstance(os_object, int)
        assert os_object > -1
        epoll = select.epoll()
        epoll.register(os_object, select.EPOLLIN)
        received_event = False
        count = 0
        trigger_error_timer = threading.Timer(1, trigger_port_error)
        trigger_error_timer.start()
        for _ in range(10):
            for fileno, ev in epoll.poll(1):
                if fileno == os_object:
                    received_event = True
                    break
            if received_event:
                break
            time.sleep(1)

        trigger_error_timer.cancel()
        # temporarily disalbe this assertion
        #assert received_event


class TestError(unittest.TestCase):
    def setUp(self):
        self.port_errors = {"errors": {"can_clear": True},
                            "first_error": {"can_clear": False},
                            "first_malformed_req": {"can_clear": False}}
        self.fme_errors = {"pcie0_errors": {"can_clear": True},
                           "warning_errors": {"can_clear": True},
                           "pcie1_errors": {"can_clear": True},
                           "gbs_errors": {"can_clear": False},
                           "bbs_errors": {"can_clear": False},
                           "next_error": {"can_clear": False},
                           "errors": {"can_clear": False},
                           "first_error": {"can_clear": False},
                           "inject_error": {"can_clear": True}}
        props = opae.fpga.properties(type=opae.fpga.ACCELERATOR)
        toks = opae.fpga.enumerate([props])
        assert toks
        self.acc_token = toks[0]
        props.type = opae.fpga.DEVICE
        toks = opae.fpga.enumerate([props])
        assert toks
        self.dev_token = toks[0]


    def test_port_errors(self):
        for err in opae.fpga.errors(self.acc_token):
            assert self.port_errors[err.name]["can_clear"] == err.can_clear
            self.port_errors.pop(err.name)
        assert not self.port_errors

    def test_fme_errors(self):
        for err in opae.fpga.errors(self.dev_token):
            assert self.fme_errors[err.name]["can_clear"] == err.can_clear
            self.fme_errors.pop(err.name)
        assert not self.fme_errors

if __name__ == "__main__":
    test = TestEvent('test_events')
    test.run()




# Copyright(c) 2017, Intel Corporation
#
# Redistribution  and  use  in source  and  binary  forms,  with  or  without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of  source code  must retain the  above copyright notice,
#  this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
#  this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
# * Neither the name  of Intel Corporation  nor the names of its contributors
#   may be used to  endorse or promote  products derived  from this  software
#   without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,  BUT NOT LIMITED TO,  THE
# IMPLIED WARRANTIES OF  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT  SHALL THE COPYRIGHT OWNER  OR CONTRIBUTORS BE
# LIABLE  FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL,  SPECIAL,  EXEMPLARY,  OR
# CONSEQUENTIAL  DAMAGES  (INCLUDING,  BUT  NOT LIMITED  TO,  PROCUREMENT  OF
# SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE,  DATA, OR PROFITS;  OR BUSINESS
# INTERRUPTION)  HOWEVER CAUSED  AND ON ANY THEORY  OF LIABILITY,  WHETHER IN
# CONTRACT,  STRICT LIABILITY,  OR TORT  (INCLUDING NEGLIGENCE  OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import shutil
import tempfile
import collections

__work_dir__ = None


def get_work_dir():
    global __work_dir__
    if not __work_dir__:
        __work_dir__ = tempfile.mkdtemp(prefix="packager_work_", dir=".")
    return __work_dir__


def delete_work_dir():
    global __work_dir__
    if __work_dir__:
        shutil.rmtree(__work_dir__)


"""
Utility function to convert input to a native type
"""


def convert_to_native_type(val):
    try:
        if isinstance(val, str) and val.startswith('0x'):
            val = int(val, 16)
        else:
            val = int(val)
        return val
    except ValueError:
        pass

    try:
        val = float(val)
        return val
    except ValueError:
        pass

    try:
        val = str(val)
        return val
    except ValueError:
        raise Exception("Cannot convert passed argument to native type!")
